/**
 * AndropodInterface - high level adb / serial communication
 *
 * @author Bernhard Wrndl-Aichriedler <bwa@xdevelop.at>, Julian Nischler  <julian@xdevelop.at>
 * @copyright Copyright 2011, Upper Austria University of Applied Sciences & \
   Julian Nischler & Bernhard Wrndl-Aichriedler
 */

#ifndef _ADB_COMMUNICATION_H
#define _ADB_COMMUNICATION_H 1

#include "Definitions.h"

//Timeout the device is allowed to answer to the connect command (in 0.1s)
//10 will result in a 1s timeout
#define ADB_COMMUNICATION_CONNECT_TIMEOUT 10

/**
 * Initialize the USB Host for adb communication
 * @param 	usbHost				USB host handle
 * @param	usbHostId			USB host id
 * @param	genericUsbHost		Generic USB host handle
 * @param	genericUsbHostId	Generic USB host id
 */
void AdbCommunicationInit(uint8 usbHostId,
	uint8 genericUsbHostId,
	uint8 usbSlaveId,
	uint8 usbSlaveAdbId,
	VOS_HANDLE uartHandle);

 /**
 * Initialize usb host and generic interface on top of it
 * @param 	usbHost				USB host handle
 * @param	usbHostId			USB host id
 * @param	genericUsbHost		Generic USB host handle
 * @param	genericUsbHostId	Generic USB host id
 */
void AdbHostInit();

/**
 * Wait for a device to be attached and connect to it
 */
bool AdbWaitAndConnect();

/**
 * Open a tcp connection tunnel to the attached and connected device
 */
bool AdbWaitAndOpenTcp();

/**
 * Close the adb connection and free resources
 */
bool AdbClose();

/**
 * Writer thread handling the communication from Uart -> Adb
 */
void AdbThreadWriter();

/**
 * Reader thread handling the communication from Adb -> Uart
 */
void AdbUartThreadReader();

/**
 * Reader thread handling the communication from Adb Slave -> Adb Host
 */
void AdbUsbThreadReader();

#endif
