/**
 * AndropodInterface - high level adb / serial communication
 *
 * @author Bernhard Julian Nischler  <julian@xdevelop.at>
 * @copyright Copyright 2011, Upper Austria University of Applied Sciences & \
 * Julian Nischler & Bernhard Wrndl-Aichriedler
 */

#include "vos.h"

/* FTDI:SHF Header Files */
#include "USB.h"
#include "USBHost.h"

#include "ioctl.h"
#include "UART.h"
#include "Debug.h"

#include "HardwareControl.h"
#include "AdbCommunication.h"

#include "USBHostGeneric.h"
/* FTDI:EHF */

/* FTDI:SDC Driver Constants */

#define VOS_DEV_UART 			0
#define VOS_DEV_USBHOST_1 		1
#define VOS_DEV_USB_GENERIC 	2
#define VOS_DEV_USBSLAVE_2 		3
#define VOS_DEV_USB_ADB_SLAVE 	4

#define VOS_NUMBER_DEVICES 		5
/* FTDI:EDC */

/* FTDI:SDH Driver Handles */
//VOS_HANDLE hUSBHOST_1; // USB Host Port 1
//VOS_HANDLE hGENERIC;
//VOS_HANDLE hUSBSLAVE_2; // USB Slave Port 2
//VOS_HANDLE hADBSLAVE;
VOS_HANDLE hUART; // UART Interface Driver
/* FTDI:EDH */


/* Declaration for IOMUx setup function */
void iomux_setup(void);

/* Main code - entry point to firmware */
void main(void)
{
	/* FTDI:SDD Driver Declarations */
	// UART Driver configuration context
	uart_context_t uartContext;
	/* UART IOCTL request block */
	common_ioctl_cb_t uart_iocb;

	/* FTDI:EDD */

	/* FTDI:SKI Kernel Initialisation */
	vos_init(50, VOS_TICK_INTERVAL, VOS_NUMBER_DEVICES);
	vos_set_clock_frequency(VOS_48MHZ_CLOCK_FREQUENCY);
	vos_set_idle_thread_tcb_size(200);
	/* FTDI:EKI */

	iomux_setup();

	hardwareControlInit();

	// Initialise UART
	uartContext.buffer_size = VOS_BUFFER_SIZE_512_BYTES;
	uart_init(VOS_DEV_UART,&uartContext);
	hUART = vos_dev_open(VOS_DEV_UART);

	// setup the UART interface
	uart_iocb.ioctl_code = VOS_IOCTL_COMMON_ENABLE_DMA;
	vos_dev_ioctl(hUART,&uart_iocb);

	/* set baud rate to 115200 baud */
	uart_iocb.ioctl_code = VOS_IOCTL_UART_SET_BAUD_RATE;
	if(getJumperStatus()) //Jumper not set -> 38400 slow baudrate (HexaCom)
	{
		uart_iocb.set.uart_baud_rate = UART_BAUD_1500000;
	}
	else
	{
		uart_iocb.set.uart_baud_rate = UART_BAUD_38400;
	}
	vos_dev_ioctl(hUART,&uart_iocb);

	/* Set the flow control scheme.  */
	uart_iocb.ioctl_code = VOS_IOCTL_UART_SET_FLOW_CONTROL;
	uart_iocb.set.param  = UART_FLOW_NONE;
	vos_dev_ioctl(hUART,&uart_iocb);

	// set data bits
	uart_iocb.ioctl_code = VOS_IOCTL_UART_SET_DATA_BITS;
	uart_iocb.set.param = UART_DATA_BITS_8;
	vos_dev_ioctl(hUART,&uart_iocb);

	// set stop bits
	uart_iocb.ioctl_code = VOS_IOCTL_UART_SET_STOP_BITS;
	uart_iocb.set.param = UART_STOP_BITS_1;
	vos_dev_ioctl(hUART,&uart_iocb);

	// set parity
	uart_iocb.ioctl_code = VOS_IOCTL_UART_SET_PARITY;
	uart_iocb.set.param = UART_PARITY_NONE;
	vos_dev_ioctl(hUART,&uart_iocb);

	// to use debugPrint with uart!
	debugInit(hUART);

	// Initialise USB Host
	AdbCommunicationInit(
		VOS_DEV_USBHOST_1,
		VOS_DEV_USB_GENERIC,
		VOS_DEV_USBSLAVE_2,
		VOS_DEV_USB_ADB_SLAVE,
		hUART);

	//Start scheduler
	vos_start_scheduler();

main_loop:
	goto main_loop;
}



