/**
 * AndropodInterface - Debug handling class
 *
 * @author Bernhard Wrndl-Aichriedler <bwa@xdevelop.at>, Julian Nischler  <julian@xdevelop.at>
 * @copyright Copyright 2011, Upper Austria University of Applied Sciences & \
   Julian Nischler & Bernhard Wrndl-Aichriedler
 */

#include "Debug.h"
#include "Convert.h"
#include "string.h"
#include "Adb.h"

VOS_HANDLE debugOutputHandle;
vos_mutex_t debugOutputMutex;

/**
 * Initialize the debug module, this has to be called before using any debugXXX
 * @param 	output	UART Handle
 */
void debugInit(VOS_HANDLE output)
{
	debugOutputHandle = output;
	vos_init_mutex(&debugOutputMutex, 0);
}

#ifdef ERROR_ENABLED
/**
 * Do a print the the output handle
 * @param	data	Zero terminated string to write
 */
void errorPrint(char *data)
{
	uint8 i = 0;
	vos_lock_mutex(&debugOutputMutex);
	while(data[i])
	{
		vos_dev_write(debugOutputHandle, data+i, 1, NULL);
		i++;
	}
	vos_unlock_mutex(&debugOutputMutex);
}

/**
 * Write a signed integer to the output handle
 * @param 	data	Data to write
 */
void errorPrintInt(int data)
{
	char adata[10]; //Maximum length of the data is set to 9
	itoa(data, adata);
	errorPrint(adata);
}

/**
 * Do a print to the Uart, without locking the write
 * @param 	data	Data to write
 */
void errorPrintUnlocked(char *data)
{
	uint8 i = 0;
	while(data[i])
	{
		vos_dev_write(debugOutputHandle, data+i, 1, NULL);
		i++;
	}
}

/**
 * Write a signed integer to the output handle, without locking the write
 * @param 	data	Data to write
 */
void errorPrintIntUnlocked(int data)
{
	char adata[10]; //Maximum length of the data is set to 9
	itoa(data, adata);
	errorPrintUnlocked(adata);
}

/**
 * Write debugging message of a adb data packet
 * @param 	data	Raw data to write
 * @param	length	Data packet
 */
void errorPrintAdbData(char *data, uint16 length)
{
	int i = 0;

	vos_lock_mutex(&debugOutputMutex);
	errorPrintUnlocked("-Data:-\n\r");

	for(i=0; i < length-1 ;i++)
	{
		vos_dev_write(debugOutputHandle, data+i, 1, NULL);
	}

	errorPrintUnlocked("\n\r-End-Data-\n\r");
	vos_unlock_mutex(&debugOutputMutex);
}

/**
 * Write the content of a adb debugging message
 * @param 	m	amessage to print
 */
void errorPrintAdbMessage(amessage *m)
{
	char data[5];

	vos_lock_mutex(&debugOutputMutex);

#ifdef ERROR_SHORT_MESSAGES
	data[3] = (m->command >> 24);
	data[2] = (m->command >> 16);
	data[1] = (m->command >> 8);
	data[0] = (m->command >> 0);
	data[4] = 0;
	errorPrintUnlocked(data);
	errorPrintUnlocked("\r\n");
#else
	errorPrintUnlocked("-Message-\r\n Command: ");

	data[3] = (m->command >> 24);
	data[2] = (m->command >> 16);
	data[1] = (m->command >> 8);
	data[0] = (m->command >> 0);
	data[4] = 0;
	errorPrintUnlocked(data);
	errorPrintUnlocked("(");
	errorPrintIntUnlocked(m->command);
	errorPrintUnlocked(")\r\n arg0: ");
	errorPrintIntUnlocked(m->arg0);
	errorPrintUnlocked("\r\n arg1: ");
	errorPrintIntUnlocked(m->arg1);
	errorPrintUnlocked("\r\n data_length: ");
	errorPrintIntUnlocked(m->data_length);
	errorPrintUnlocked("\r\n data_check: ");
	errorPrintIntUnlocked(m->data_check);
	errorPrintUnlocked("\r\n magic: ");
	errorPrintIntUnlocked(m->magic);

	errorPrintUnlocked("\r\n-End-Message-\r\n");
#endif
	vos_unlock_mutex(&debugOutputMutex);
}

#endif

