/**
 * AndropodInterface - Debug handling class
 *
 * @author Bernhard Wrndl-Aichriedler <bwa@xdevelop.at>, Julian Nischler  <julian@xdevelop.at>
 * @copyright Copyright 2011, Upper Austria University of Applied Sciences & \
   Julian Nischler & Bernhard Wrndl-Aichriedler
 */

#ifndef _DEBUG_H
#define _DEBUG_H 1

#include "vos.h"
#include "Adb.h"

/**
 * Enable debugging
 */
//#define DEBUG_ENABLED 1
//#define ERROR_ENABLED 1

#define ERROR_SHORT_MESSAGES 1

/**
 * Initialize the debug module, this has to be called before using any debugXXX
 * @param 	output	UART Handle
 */
void debugInit(VOS_HANDLE output);

/**
 * Do a print the the output handle
 * @param	data	Zero terminated string to write
 */
#ifdef ERROR_ENABLED
	void errorPrint(char *data);
#else
	#define errorPrint(x) while(false)
#endif

/**
 * Do a print the the output handle
 * @param	data	Zero terminated string to write
 */
#ifdef DEBUG_ENABLED
	#define debugPrint(x) errorPrint(x)
#else
	#define debugPrint(x) while(false)
#endif

/**
 * Write a signed integer to the output handle
 * @param 	data	Data to write
 */
#ifdef ERROR_ENABLED
	void errorPrintInt(int data);
#else
	#define errorPrintInt(x) while(false)
#endif

/**
 * Write a signed integer to the output handle
 * if debugging is enabled -> method used to kill string that when compiling
 * without debugging
 * @param 	data	Data to write
 */
#ifdef DEBUG_ENABLED
	#define debugPrintInt(x) errorPrintInt(x)
#else
	#define debugPrintInt(x) while(false)
#endif

/**
 * Print a adbMessage structure to the debug
 */
#ifdef ERROR_ENABLED
	void errorPrintAdbMessage(amessage *m);
#else
	#define errorPrintAdbMessage(x) while(false)
#endif

/**
 * Print a adbMessage structure to the debug
 */
#ifdef DEBUG_ENABLED
	#define debugPrintAdbMessage(x) errorPrintAdbMessage(x)
#else
	#define debugPrintAdbMessage(x) while(false)
#endif

/**
 * Print the data of a adbMessage specially formated
 */
#ifdef ERROR_ENABLED
	void errorPrintAdbData(char *data, uint16 length);
#else
	#define errorPrintAdbData(x, y) while(false)
#endif

/**
 * Print the data of a adbMessage specially formated
 */
#ifdef DEBUG_ENABLED
	#define debugPrintAdbData(x,y) errorPrintAdbData(x,y)
#else
	#define debugPrintAdbData(x,y) while(false)
#endif

#endif
