/**
 * AndropodInterface - Simplified IO control
 *
 * @author Bernhard Wrndl-Aichriedler <bwa@xdevelop.at>, Julian Nischler  <julian@xdevelop.at>
 * @copyright Copyright 2011, Upper Austria University of Applied Sciences & \
   Julian Nischler & Bernhard Wrndl-Aichriedler
 */

#include "DigitalIO.h";
#include "IOMux.h"

/**
 * Definitions of the GPIO registers of port A
 */
#define VII_GPIO_SYS_CNTRL_1 0x180
#define VII_GPIO_DATA_TX_PA_1  0x186
#define VII_GPIO_DATA_RX_PA_1  0x18b
#define VII_GPIO_CNTRL_PORTA_1 0x181
#define VII_GPIO_INT_REG_EN_1 0x19a

/**
 * Definitions of the control registers on port A
 */
port vII_gpio_sys_cntrl_1 @ VII_GPIO_SYS_CNTRL_1;
port vII_gpio_data_tx_pa_1 @ VII_GPIO_DATA_TX_PA_1;
port vII_gpio_data_rx_pa_1 @ VII_GPIO_DATA_RX_PA_1;
port vII_gpio_cntrl_porta_1 @ VII_GPIO_CNTRL_PORTA_1;
port vII_gpio_int_reg_en_1 @ VII_GPIO_INT_REG_EN_1;

/**
 * Definition of serveral control bits
 */
#define BIT_INT_4_IEN 4
#define MASK_GPIO_EN ( 1 << 1)

/*
 * Define DigitalIoPinValue globally in order for assembler to access it
 */
unsigned char DigitalIoPinValue;

/*
 * Define DigitalIoPortValue globally in order for assembler to access it
 */
unsigned char DigitalIoPortValue;

/*
 * Define DigitalIoTemp globally in order for assembler to access it
 */
unsigned char DigitalIoTemp;

/*
 * Define DigitalIoBitMask globally in order for assembler to access it
 */
unsigned char DigitalIoBitMask;

/**
 * Set the value of an Io pin
 * @param	pin		Pin number on port A
 * @param	value	Value of pin high/low
 */
void SetIoValue(uint8 pin, bool value)
{
    DigitalIoBitMask = (1 << pin);

    if (value == true)
    {
        asm
        {
            INPORT      vII_gpio_data_tx_pa_1 DigitalIoPortValue
            OR8         DigitalIoPortValue DigitalIoBitMask
            OUTPORT     vII_gpio_data_tx_pa_1 DigitalIoPortValue
		}
    }
    else /* value == LOW */
    {
        asm
        {
            INPORT      vII_gpio_data_tx_pa_1 DigitalIoPortValue
            INV8        DigitalIoTemp DigitalIoBitMask
			CPY8		DigitalIoBitMask DigitalIoTemp
            AND8        DigitalIoPortValue DigitalIoBitMask
            OUTPORT     vII_gpio_data_tx_pa_1 DigitalIoPortValue
		}
    }
}

/**
 * Read the value of an Io pin
 * @param	pin		Pin number on port A
 * @return	pin value
 */
bool GetIoValue(uint8 pin)
{
    DigitalIoBitMask = (1 << pin);

	asm
 	{
		INPORT			vII_gpio_data_rx_pa_1	DigitalIoPortValue
        AND8            DigitalIoPortValue DigitalIoBitMask
        CMP8            DigitalIoPortValue $0x00
        JZ              @ReturnLow
        @ReturnHigh:
        LD8             DigitalIoPinValue $0x01
        JUMP            @End
        @ReturnLow:
        LD8             DigitalIoPinValue $0x00
        @End:
    }
	return DigitalIoPinValue;
}

/**
 * Set the io mode of a pin
 * @param	pin		Pin on port A
 * @param	output	true sets the pin as output, false sets it as input
 */
void SetIoMode(uint8 pin, bool output)
{
    DigitalIoBitMask = (1<<pin);

    if (!output)
    {
		vII_gpio_int_reg_en_1.BIT_INT_4_IEN = 0;
        asm
		{
            INPORT      vII_gpio_cntrl_porta_1 DigitalIoPortValue
            INV8        DigitalIoTemp DigitalIoBitMask
			CPY8		DigitalIoBitMask DigitalIoTemp
            AND8        DigitalIoPortValue DigitalIoBitMask
            OUTPORT     vII_gpio_cntrl_porta_1 DigitalIoPortValue
		}
	}
    else /* mode == OUTPUT */
    {
		vII_gpio_int_reg_en_1.BIT_INT_4_IEN = 0;
		asm
		{
	            INPORT      vII_gpio_cntrl_porta_1 DigitalIoPortValue
	            OR8         DigitalIoPortValue DigitalIoBitMask
	            OUTPORT     vII_gpio_cntrl_porta_1 DigitalIoPortValue
        }
    }
	vII_gpio_sys_cntrl_1 = MASK_GPIO_EN;
}


