/**
 * AndropodInterface - basic hardware control functions
 *
 * @author Bernhard Wrndl-Aichriedler <bwa@xdevelop.at>, Julian Nischler  <julian@xdevelop.at>
 * @copyright Copyright 2011, Upper Austria University of Applied Sciences & \
   Julian Nischler & Bernhard Wrndl-Aichriedler
 */
#ifndef _HARDWARE_CONTROL
#define _HARDWARE_CONTROL 1

#include "vos.h"
#include "Definitions.h"

/**
 * Definition of the timeout till the power of the USB port is reenabled in a
 * condition
 */
#define HARDWARE_CONTROL_OVERCURRENT_TIMEOUT 49 //5 sec timeout -> just to get currect blink interval

/**
 * Initialize the hardware control module
 * @param gpioHandle	Pointer to the GPIO handle
 */
void hardwareControlInit();

/**
 * Read the USB Host power switch overcurrent signal @ IOBUS10
 */
bool getOvercurrentStatus();

/**
 * Enable the USB Host power switch @ IOBUS11
 */
void setCurrentSwitch(bool enabled);

/**
 * Overcurrent supervising thread used to monitor the overcurrent
 * condition for the USB Host port.
 * If an overcurrent condition is raised, the power will be cut of, and
 * reenabled after HARDWARE_CONTROL_OVERCURRENT_TIMEOUT loops
 */
void overcurrentSupervisorThread();

/**
 * Initialize the watchdog timer of the VNC32
 */
void watchdogInit();

/**
 * Do a watchdog reset
 */
void watchdogReset();

/**
 * Reset the whole system
 */
void systemReset();

/**
 * Check if AndropodInterface's device port is plugged to a host
 * this is done by simple voltage sensing on the VBUS pin @ IOBUS9
 */
bool pluggedToUsbHost();


/**
 * Set the status of the status led @ IOBUS2
 * @param enabled	LED enabled
 */
void setStatusLed(bool enabled);

/**
 * Set the status of the activity led @ IOBUS1
 * @param enabled	LED enabled
 */
void setActivityLed(bool enabled);

/**
 * Read the status of the jumper
 */
bool getJumperStatus();


#endif
